"""Main module."""
__version__ = "0.1.1"


# from config import _T_SURROUNDING, _P_SURROUNDING, _RESULTS_DIR 
# import src.models as models
# import src.utils as utils
# import src.helpers as helpers

# from src.models.fluids import fluid_props as fprop
# from src.models.coupled import heat_pump_simple_v2 as hp_simple
# from src.models.components import compressor_simple, throttle_simple
# from src.models.components import heat_exchanger_thermo_v2 as hex_th
# from src.utils import exergy_loss as exlo
# from src.helpers import file_copy


# import src.models.fluids.fluid_props as fprop
# from src.models.coupled import heat_pump_simple_v2 as hp_simple
# from src.models.components import compressor_simple, throttle_simple
# from src.models.components import heat_exchanger_thermo_v2 as hex_th
# from src.utils import exergy_loss as exlo

#_T_SURROUNDING = 288.15 # K



